import { render } from '@testing-library/react';
import TiledLayer from './TiledLayer';
import { TileFactory } from './types';

describe('The TiledLayer component ', () => {
  beforeEach(() => {
    globalThis.cacheForTiledLayer = new Map();
  });

  test('exposes the tiles needed per its viewport', () => {
    const { baseElement } = render(
      <svg>
        <TiledLayer
          keyObject={{ provider: 'osm', zoomLevel: 10, x: 5, y: 8 }}
          viewPort={{ topLeft: { x: 1, y: 2 }, bottomRight: { x: 3, y: 2 } }}
        />
      </svg>
    );
    //    screen.debug();
    expect(baseElement).toMatchInlineSnapshot(`
<body>
  <div>
    <svg>
      <g
        transform="translate(1, 2)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/6/10.png"
            width="1"
          />
        </g>
      </g>
      <g
        transform="translate(2, 2)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/7/10.png"
            width="1"
          />
        </g>
      </g>
      <g
        transform="translate(3, 2)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/8/10.png"
            width="1"
          />
        </g>
      </g>
    </svg>
  </div>
</body>
`);
  });

  test('adds more tiles when its viewport is updated without removing the previous ones', () => {
    const { baseElement, rerender } = render(
      <svg>
        <TiledLayer
          keyObject={{ provider: 'osm', zoomLevel: 10, x: 5, y: 8 }}
          viewPort={{ topLeft: { x: 1, y: 2 }, bottomRight: { x: 3, y: 2 } }}
        />
      </svg>
    );
    rerender(
      <svg>
        <TiledLayer
          keyObject={{ provider: 'osm', zoomLevel: 10, x: 5, y: 8 }}
          viewPort={{ topLeft: { x: 5, y: 0 }, bottomRight: { x: 5, y: 2 } }}
        />
      </svg>
    );
    //    screen.debug();
    expect(baseElement).toMatchInlineSnapshot(`
<body>
  <div>
    <svg>
      <g
        transform="translate(1, 2)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/6/10.png"
            width="1"
          />
        </g>
      </g>
      <g
        transform="translate(2, 2)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/7/10.png"
            width="1"
          />
        </g>
      </g>
      <g
        transform="translate(3, 2)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/8/10.png"
            width="1"
          />
        </g>
      </g>
      <g
        transform="translate(5, 0)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/10/8.png"
            width="1"
          />
        </g>
      </g>
      <g
        transform="translate(5, 1)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/10/9.png"
            width="1"
          />
        </g>
      </g>
      <g
        transform="translate(5, 2)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/10/10.png"
            width="1"
          />
        </g>
      </g>
    </svg>
  </div>
</body>
`);
  });
  test('is not reinitialized if its key isObject updated', () => {
    const { baseElement, rerender } = render(
      <svg>
        <TiledLayer
          keyObject={{ provider: 'osm', zoomLevel: 10, x: 5, y: 8 }}
          viewPort={{ topLeft: { x: 1, y: 2 }, bottomRight: { x: 3, y: 2 } }}
        />
      </svg>
    );
    rerender(
      <svg>
        <TiledLayer
          keyObject={{ provider: 'osm', zoomLevel: 10, x: 4, y: 10 }}
          viewPort={{ topLeft: { x: 5, y: 0 }, bottomRight: { x: 5, y: 2 } }}
        />
      </svg>
    );
    //    screen.debug();
    expect(baseElement).toMatchInlineSnapshot(`
<body>
  <div>
    <svg>
      <g
        transform="translate(1, 2)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/6/10.png"
            width="1"
          />
        </g>
      </g>
      <g
        transform="translate(2, 2)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/7/10.png"
            width="1"
          />
        </g>
      </g>
      <g
        transform="translate(3, 2)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/8/10.png"
            width="1"
          />
        </g>
      </g>
      <g
        transform="translate(5, 0)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/9/10.png"
            width="1"
          />
        </g>
      </g>
      <g
        transform="translate(5, 1)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/9/11.png"
            width="1"
          />
        </g>
      </g>
      <g
        transform="translate(5, 2)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/9/12.png"
            width="1"
          />
        </g>
      </g>
    </svg>
  </div>
</body>
`);
  });
  test('Also works with negative coordinates', () => {
    const { baseElement } = render(
      <svg>
        <TiledLayer
          keyObject={{ provider: 'osm', zoomLevel: 10, x: 5, y: 8 }}
          viewPort={{ topLeft: { x: -3, y: -1 }, bottomRight: { x: -2, y: 2 } }}
        />
      </svg>
    );
    //    screen.debug();
    expect(baseElement).toMatchInlineSnapshot(`
<body>
  <div>
    <svg>
      <g
        transform="translate(-3, -1)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/2/7.png"
            width="1"
          />
        </g>
      </g>
      <g
        transform="translate(-2, -1)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/3/7.png"
            width="1"
          />
        </g>
      </g>
      <g
        transform="translate(-3, 0)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/2/8.png"
            width="1"
          />
        </g>
      </g>
      <g
        transform="translate(-2, 0)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/3/8.png"
            width="1"
          />
        </g>
      </g>
      <g
        transform="translate(-3, 1)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/2/9.png"
            width="1"
          />
        </g>
      </g>
      <g
        transform="translate(-2, 1)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/3/9.png"
            width="1"
          />
        </g>
      </g>
      <g
        transform="translate(-3, 2)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/2/10.png"
            width="1"
          />
        </g>
      </g>
      <g
        transform="translate(-2, 2)"
      >
        <g>
          <image
            height="1"
            href="https://tile.openstreetmap.org/10/3/10.png"
            width="1"
          />
        </g>
      </g>
    </svg>
  </div>
</body>
`);
  });
});
